<?php
/*
Plugin Name: 		Codepress Menu
Plugin URI:         http://wordpress.org/extend/plugins/codepress-menu/
Version: 			2.2.1
Description: 		A more flexible take on displaying a WordPress menu in your theme
Author: 			Codepress
License:			GPLv2

Copyright 2013  Codepress  info@codepress.nl

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License version 2 as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class Codepress_Walker_Nav_Menu extends Walker_Nav_Menu {

    /**
	 * Indents any object as long as it has a unique id and that of its parent.
	 *
     * @since 2.1
	 */
	protected function indent( array $items, $root, $parent, $id ) {
        $indent = array();

		foreach( $items as $k => $item ) {
			if ( $item->$parent == $root ) {
				unset( $items[$k] );

                $item->children = $this->indent( $items, $item->$id, $parent, $id );
                $indent[]		= $item;
			}
		}

		return $indent;
	}

    /**
     * Only return a level from a indented list of items.
     *
     * @since 2.0
     */
    protected function level( $items, $level, $reached = 1, $currentBranch = false)  {

        foreach ( $items as $item ) {

            // check for current or ancestor
			if ( $item->current || $item->current_item_ancestor || $currentBranch ) {

                // catch top level
                if ( $level == 1 && $reached == 1 )
                    return array( $item );

                // catch all other levels
                if ( $reached + 1 == $level && ! empty( $item->children ) )
                    return $item->children;

                if ( ! empty( $item->children ) )
                   return $this->level( $item->children, $level, $reached + 1, true );
			}
		}

		return array();
    }

    /**
	 * Unset children exceeding max depth.
	 *
     * @since 1.0
	 */
	protected function depth( $items, $depth, $level = 1 ) {

		foreach( $items as $k => $item ) {
            // unset the children, or go a level deeper
			if ( $depth == $level )
				unset( $items[$k]->children );
			elseif ( ! empty( $item->children ) )
				$items[$k]->children = $this->depth( $item->children, $depth, $level + 1 );
		}

        return $items;
	}

    /**
     * Adds classes to the first and last item of each level.
     *
     * @since 2.1
     */
    protected function mark_first_and_last( array $items ) {
        $i = 0;

        foreach( $items as $item ) {
            if ( $i == 0 ) {
                $item->classes[] = 'first';
            }

            $i++;

            if ( $i == count( $items ) ) {
                $item->classes[] = 'last';
            }

            if ( ! empty( $item->children ) ) {
                $item->children = $this->mark_first_and_last( $item->children );
            }
        }

        return $items;
    }

    /**
	 * Sanitize classes by removing some of them and renaming others.
	 *
     * @since 2.1.1
	 */
	protected function sanitize_classes( $items, $args ) {

        foreach( $items as $k => $item ) {

            // always allow custom assigned classes
            $filtered = (array) get_post_meta( $item->ID, '_menu_item_classes', true );
            $allowed  = array();

            if ( $args->classes == 'minimal' || $args->classes == 'simple' ) {
                $allowed = array(
                    'current-menu-item'     => 'current',
                    'current-menu-parent'   => 'current-parent',
                    'current-menu-ancestor' => 'current-ancestor',
                );
            }

            if ( $args->classes == 'sanitize' ) {
                $allowed = array(
                    'current-menu-item',
                    'current-menu-parent',
                    'current-menu-ancestor',
                );
            }

            if ( $args->classes == 'simple' || $args->classes == 'sanitize' )
                $allowed = array_merge( $allowed, array( 'first', 'last' ) );

            $allowed = apply_filters( 'codepress_menu_filter_classes', $allowed, $item, $args );

            // check for class that is allowed, but needs transformation
            foreach ( $item->classes as $class ) {
                if ( array_key_exists( $class, $allowed ) )
                    $filtered[] = $allowed[$class];
                 elseif ( in_array( $class, $allowed ) )
                    $filtered[] = $class;
            }

            $items[$k]->classes = array_filter( $filtered );;
        }

		return $items;
	}

    /**
     * Walkers have a walk method, makes sense.
     *
     * @since 2.1
     */
    public function walk( $items, $depth, $args ) {

        $items = $this->indent( $items, 0, $this->db_fields['parent'], $this->db_fields['id'] );

        if ( $args->level )
            $items = $this->level( $items, $args->level );

        $items = $this->depth( $items, $depth );

        if ( $args->classes ) {
            $items = $this->mark_first_and_last( $items );
            $items = $this->sanitize_classes( $items, $args );
        }

        $items = apply_filters( 'codepress_menu_filter', $items );

        return $this->get_menu( $items, $args );
    }

    /**
	 * Get the menu string.
	 *
     * @return string
     * @since 1.0
	 */
    public function get_menu( $items, $args, $level = 1 ) {
        $output = '';

        if ( $level > 1)
            $this->start_lvl( $output, 0, $args );

        foreach( $items as $item ) {
            $this->start_el( $output, $item, 0, $args );

            if ( ! empty( $item->children ) )
                $output .= $this->get_menu( $item->children, $args, $level + 1);

            $this->end_el( $output, $item, 0, $args );
        }

        if ( $level > 1 )
            $this->end_lvl( $output, 0, $args );

        return $output;
    }
}

/**
 * Init Codepress Menu.
 *
 * @uses apply_filters() To ignore certain theme_locations.
 * @since 1.0
 */
function codepress_menu_init( $args ) {
    $ignore = apply_filters( 'codepress_ignore_theme_locations', array() );

    if ( ! empty( $ignore ) && in_array( $args['theme_location'], $ignore ) )
        return $args;

	// check if the theme location requested exists and is set
	if ( ! empty( $args['theme_location'] ) ) {
		$found = false;

		foreach( get_nav_menu_locations() as $theme_location => $id ) {
			if ( $theme_location == $args['theme_location'] && $id > 0 )
				$found = true;
		}

		if ( ! $found )
			return $args;
	}

    $defaults = array(
        'level'     => 0,
        'classes'   => false,
        'walker'    => new Codepress_Walker_Nav_Menu,
    );

    foreach( $defaults as $option => $default ) {
        if ( empty( $args[$option] ) )
            $args[$option] = $default;
    }

    return $args;
}
add_filter( 'wp_nav_menu_args', 'codepress_menu_init' );